package com.winhc.dataworks.flow.touch;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.helospark.lightdi.LightDi;
import com.helospark.lightdi.LightDiContext;
import com.helospark.lightdi.annotation.Autowired;
import com.helospark.lightdi.annotation.Service;
import com.winhc.dataworks.flow.touch.bean.*;
import com.winhc.dataworks.flow.touch.configuration.SchemaInit;
import com.winhc.dataworks.flow.touch.service.TouchService;
import com.winhc.dataworks.flow.touch.utils.DingUtils;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 10:45
 * @Description:
 */
@Slf4j
@Service
public class Main {
    @Autowired
    private TouchService touchService;

    @Autowired
    private DingUtils dingUtils;


    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("请输入调度的 bizDate");
            System.exit(-9999);
        }
        LightDiContext context = LightDi.initContextByPackage(Main.class.getPackage().getName());
        Main bean = context.getBean(Main.class);
        bean.start(args[0]);
    }

    @SneakyThrows
    private void start(String bizDate) {
        log.info("start");
        LocalDateTime start = LocalDateTime.now();
        List<DataWorksFlowJob> jobs = SchemaInit.LIST;
        List<TaskInfo> collect = jobs.stream()
                .flatMap(dataWorksFlowJob -> {
                    List<DataWorksFlowTask> task = dataWorksFlowJob.getTask();
                    return task
                            .stream()
                            .map(dataWorksFlowTask -> {
                                TaskParam build = TaskParam.builder()
                                        .projectName(dataWorksFlowJob.getProject())
                                        .bizDate(bizDate)
                                        .flowName(dataWorksFlowJob.getFlow())
                                        .nodeParam(dataWorksFlowTask.toNodeParam())
                                        .build();
                                CreateManualDagResponse r = touchService.touch(build);

                                return TaskInfo.builder().project(dataWorksFlowJob.getProject())
                                        .flow(dataWorksFlowJob.getFlow())
                                        .taskName(dataWorksFlowTask.getTaskName())
                                        .dagId(r.getReturnValue()).build();
                            });
                }).collect(Collectors.toList());

        int totalTask = collect.size();

        Set<TaskInfo> end = new HashSet<>();
        TimedCache<TaskInfo, String> timedCache = CacheUtil.newTimedCache(300 * 1000);
        int i = 0;
        while (true) {
            int awaitTask = 0;
            int successTask = 0;
            int failedTask = 0;

            boolean flag = true;
            List<String> empty = new ArrayList<>();

            for (TaskInfo taskInfo : collect) {
                if (end.contains(taskInfo)) {
                    continue;
                }
                Map<String, TaskFlowEnum> query = touchService.query(taskInfo.getProject(), taskInfo.getDagId());
                Map<String, List<String>> status = query.entrySet().stream()
                        .map(e -> {
                            Entry<String, String> entry = new Entry<>();
                            if (e.getValue().equals(TaskFlowEnum.SUCCESS) || e.getValue().equals(TaskFlowEnum.FAILURE)) {
                                entry.setKey(e.getValue().getMsg());
                            } else {
                                entry.setKey("运行中");
                            }
                            entry.setValue(e.getKey());
                            return entry;
                        }).collect(Collectors.groupingBy(Entry::getKey, Collectors.mapping(Entry::getValue, Collectors.toList())));
                List<String> success = status.getOrDefault(TaskFlowEnum.SUCCESS.getMsg(), empty);
                List<String> failure = status.getOrDefault(TaskFlowEnum.FAILURE.getMsg(), empty);
                List<String> await = status.getOrDefault("运行中", empty);


                if (failure.size() != 0) {
                    failedTask++;
                    log.error("failure node:{} ", failure);
                    DingMsg error = new DingMsg("任务失败", taskInfo.getProject(), taskInfo.getFlow(), String.join(",", failure), TaskFlowEnum.FAILURE.getMsg());
                    dingUtils.send(error);
                } else {
                    if (await.size() != 0) {
                        awaitTask++;
                    } else {
                        successTask++;
                    }
                }


                flag = flag && (await.size() == 0 || failure.size() != 0);

                if (await.size() == 0 || failure.size() != 0) {
                    end.add(taskInfo);
                } else {
                    LocalDateTime now = LocalDateTime.now();
                    Duration duration = Duration.between(start, now);
                    long l = duration.toHours();
                    if (l >= 2) {
                        if (!timedCache.containsKey(taskInfo) && i <= 6) {
                            //超两小时
                            i++;
                            DingMsg error = new DingMsg("【" + i + "】任务长时间未结束", taskInfo.getProject(), taskInfo.getFlow(), String.join(",", failure), TaskFlowEnum.RUNNING.getMsg());
                            dingUtils.send(error);
                            timedCache.put(taskInfo, "1");
                        }
                    }
                }
            }

            log.info("\nawait task:{}\ntotal task:{}\nsuccess task:{}\nfailure task:{}\n"
                    , awaitTask
                    , totalTask
                    , successTask
                    , failedTask
            );

            if (flag) {
                break;
            }
            Thread.sleep(10000);
        }
        log.info("end");
    }
}
